@@ -51,10 +51,11 @@ from shutil import rmtree
 import textwrap
 import typing
 
-# numpy and rpy2 imports
+import rpy2.rinterface_lib.callbacks
 
 import rpy2.rinterface as ri
-import rpy2.rinterface_lib.callbacks
+
+import rpy2.rinterface_lib.openrlib
 import rpy2.robjects as ro
 import rpy2.robjects.packages as rpacks
 from rpy2.robjects.lib import grdevices
@@ -103,7 +104,7 @@ from IPython.core.magic_arguments import (argument,  # type: ignore
 
 template_converter = get_conversion()
 
-DEVICES_STATIC_RASTER = {'png', 'jpeg'}
+DEVICES_STATIC_RASTER: typing.Set[str] = {'png', 'jpeg'}
 DEVICES_STATIC = DEVICES_STATIC_RASTER | {'svg'}
 DEVICES_SUPPORTED = DEVICES_STATIC | {'X11'}
 
@@ -373,21 +374,40 @@ class RMagics(Magics):
         withVisible is a LISPy R function).
         """
         with contextlib.ExitStack() as stack:
+            obj_in_module = (rpy2.rinterface_lib
+                             .callbacks.obj_in_module)
             if self.cache_display_data:
-                stack.enter(
-                    rpy2.rinterface_lib
-                    .callbacks.obj_in_module(rpy2.rinterface_lib.callbacks,
-                                             'consolewrite_print',
-                                             self.write_console_regular)
+                stack.enter_context(
+                    obj_in_module(
+                        rpy2.rinterface_lib.callbacks,
+                        'consolewrite_print',
+                        self.write_console_regular
+                    )
                 )
+            stack.enter_context(
+                obj_in_module(rpy2.rinterface_lib.callbacks,
+                              'consolewrite_warnerror',
+                              self.write_console_regular)
+            )
+            stack.enter_context(
+                obj_in_module(
+                    rpy2.rinterface_lib.callbacks,
+                    '_WRITECONSOLE_EXCEPTION_LOG',
+                    '%s')
+            )
             try:
                 # Need the newline in case the last line in code is a comment.
-                value, visible = ro.r("withVisible({%s\n})" % code)
+                r_expr = ri.parse(code)
+                value, visible = ri.evalr_expr_with_visible(
+                    r_expr
+                )
             except (ri.embedded.RRuntimeError, ValueError) as exception:
                 # Otherwise next return seems to have copy of error.
                 warning_or_other_msg = self.flush()
                 raise RInterpreterError(code, str(exception),
                                         warning_or_other_msg)
+            finally:
+                ro._print_deferred_warnings()
             text_output = self.flush()
             return text_output, value, visible[0]
 
@@ -924,14 +944,29 @@ class RMagics(Magics):
                 text_output += text_result
                 if visible:
                     with contextlib.ExitStack() as stack:
+                        obj_in_module = (rpy2.rinterface_lib
+                                         .callbacks
+                                         .obj_in_module)
                         if self.cache_display_data:
                             stack.enter_context(
-                                rpy2.rinterface_lib
-                                .callbacks
-                                .obj_in_module(rpy2.rinterface_lib
-                                               .callbacks,
-                                               'consolewrite_print',
-                                               self.write_console_regular))
+                                obj_in_module(rpy2.rinterface_lib
+                                              .callbacks,
+                                              'consolewrite_print',
+                                              self.write_console_regular)
+                            )
+                        stack.enter_context(
+                            obj_in_module(
+                                rpy2.rinterface_lib.callbacks,
+                                'consolewrite_warnerror',
+                                self.write_console_regular
+                            )
+                        )
+                        stack.enter_context(
+                            obj_in_module(
+                                rpy2.rinterface_lib.callbacks,
+                                '_WRITECONSOLE_EXCEPTION_LOG',
+                                '%s')
+                        )
                         cell_display(result, args)
                         text_output += self.flush()
 
