@@ -1,13 +1,17 @@
-from collections import namedtuple
-from rpy2.robjects.packages import importr, data, WeakPackage
+from rpy2.robjects.packages import (importr,
+                                    WeakPackage)
+from rpy2.robjects.lib import dplyr
 import warnings
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")
     tidyr = importr('tidyr', on_conflict="warn")
-    
-TARGET_VERSION = '0.7.2'
-if tidyr.__version__ != TARGET_VERSION:
-    warnings.warn('This was designed againt tidyr version %s but you have %s' % (TARGET_VERSION, tidyr.__version__))
+
+TARGET_VERSION = '1.2.'
+
+if not tidyr.__version__.startswith(TARGET_VERSION):
+    warnings.warn(
+        'This was designed againt tidyr versions starting with %s '
+        'but you have %s' % (TARGET_VERSION, tidyr.__version__))
 
 tidyr = WeakPackage(tidyr._env,
                     tidyr.__rname__,
@@ -16,18 +20,9 @@ tidyr = WeakPackage(tidyr._env,
                     on_conflict="warn",
                     version=tidyr.__version__,
                     symbol_r2python=tidyr._symbol_r2python,
-                    symbol_check_after=tidyr._symbol_check_after)
+                    symbol_resolve=tidyr._symbol_resolve)
 
 
-    
-from rpy2.robjects.lib import dplyr
-
-class DataFrame(dplyr.DataFrame):
-    pass
-
-DataFrame.summarize = dplyr._wrap(dplyr.summarize, DataFrame)
-DataFrame.summarise = DataFrame.summarize
-
 def _wrap(rfunc):
     def func(dataf, *args, **kwargs):
         cls = type(dataf)
@@ -35,6 +30,11 @@ def _wrap(rfunc):
         return cls(res)
     return func
 
-DataFrame.gather = _wrap(tidyr.gather_)
-DataFrame.spread = _wrap(tidyr.spread_)
 
+class DataFrame(dplyr.DataFrame):
+    gather = _wrap(tidyr.gather)
+    spread = _wrap(tidyr.spread)
+    summarize = dplyr._wrap(dplyr.summarize, None)
+
+
+DataFrame.summarise = DataFrame.summarize
